/*
 * jSite-next - CoreListenerSupport.java -
 * Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.core;

import java.util.logging.Level;
import java.util.logging.Logger;

import net.pterodactylus.util.event.ListenerSupport;

/**
 * TODO
 *
 * @author David Roden &lt;droden@gmail.com&gt;
 */
public class CoreListenerSupport extends ListenerSupport<CoreListener> {

	/** The logger. */
	private static final Logger logger = Logger.getLogger(CoreListenerSupport.class.getName());

	/**
	 * Notifies all listeners that the projects were loaded successfully.
	 *
	 * @param directory
	 *            The directory the projects were loaded from
	 */
	public void fireLoadingProjectsDone(String directory) {
		logger.log(Level.FINEST, "firing loadingProjectsDone event with [directory=" + directory + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.loadingProjectsDone(directory);
		}
	}

	/**
	 * Notifies all core listeners that loading the projects from the given
	 * directory has failed.
	 *
	 * @param directory
	 *            The directory the projects were tried to load from
	 * @param throwable
	 *            The exception that occured when loading projects
	 */
	public void fireLoadingProjectsFailed(String directory, Throwable throwable) {
		logger.log(Level.FINEST, "firing loadingProjectsFailed event with [directory=" + directory + ",throwable=" + throwable + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.loadingProjectsFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all listeners that the projects were successfully saved.
	 *
	 * @param directory
	 *            The directory the projects were saved to
	 */
	public void fireSavingProjectsDone(String directory) {
		logger.log(Level.FINEST, "firing savingProjectsDone event with [directory=" + directory + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.savingProjectsDone(directory);
		}
	}

	/**
	 * Notifies all listeners that the projects could not be saved.
	 *
	 * @param directory
	 *            The directory the projects were to be saved to
	 * @param throwable
	 *            The exception that occured when saving the projects
	 */
	public void fireSavingProjectsFailed(String directory, Throwable throwable) {
		logger.log(Level.FINEST, "firing savingProjectsFailed event with [directory=" + directory + ",throwable=" + throwable + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.savingProjectsFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all listeners that a project was added.
	 *
	 * @param project
	 *            The project that was added
	 */
	public void fireProjectAdded(Project project) {
		logger.log(Level.FINEST, "firing projectAdded event with [project=" + project + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.projectAdded(project);
		}
	}

	/**
	 * Notifies all listeners that a project was cloned.
	 *
	 * @param clonedProject
	 *            The project that was cloned
	 * @param projectClone
	 *            The clone of the project
	 */
	public void fireProjectCloned(Project clonedProject, Project projectClone) {
		logger.log(Level.FINEST, "firing projectCloned event with [clonedProject=" + clonedProject + ",projectClone=" + projectClone + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.projectCloned(clonedProject, projectClone);
		}
	}

	/**
	 * Notifies all listeners that a project was removed.
	 *
	 * @param project
	 *            The project that was removed
	 */
	public void fireProjectRemoved(Project project) {
		logger.log(Level.FINEST, "firing projectRemoved event with [project=" + project + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.projectRemoved(project);
		}
	}

	/**
	 * Notifies all listeners that the nodes were successfully loaded.
	 *
	 * @param directory
	 *            The directory the nodes were loaded from
	 */
	public void fireLoadingNodesDone(String directory) {
		logger.log(Level.FINEST, "firing loadingNodesDone event with [directory=" + directory + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.loadingNodesDone(directory);
		}
	}

	/**
	 * Notifies all listeners that loading the nodes has failed.
	 *
	 * @param directory
	 *            The directory the nodes were loaded from
	 * @param throwable
	 *            The exception that occured while loading the nodes
	 */
	public void fireLoadingNodesFailed(String directory, Throwable throwable) {
		logger.log(Level.FINEST, "firing loadingNodesFailed event with [directory=" + directory + ",throwable=" + throwable + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.loadingNodesFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all listeners that the nodes were saved successfully.
	 *
	 * @param directory
	 *            The directory the nodes were saved to
	 */
	public void fireSavingNodesDone(String directory) {
		logger.log(Level.FINEST, "firing savingNodesDone event with [directory=" + directory + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.savingNodesDone(directory);
		}
	}

	/**
	 * Notifies all listeners that saving the nodes has failed.
	 *
	 * @param directory
	 *            The directory the nodes were saved to
	 * @param throwable
	 *            The exception that occured while saving the nodes
	 */
	public void fireSavingNodesFailed(String directory, Throwable throwable) {
		logger.log(Level.FINEST, "firing savingNodesFailed event with [directory=" + directory + ",throwable=" + throwable + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.savingProjectsFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all core listeners that the core has loaded and is ready to run.
	 */
	public void fireCoreLoaded() {
		logger.log(Level.FINEST, "firing coreLoaded event");
		for (CoreListener coreListener : getListeners()) {
			coreListener.coreLoaded();
		}
	}

	/**
	 * Notifies all listeners that the core was stopped.
	 */
	public void fireCoreStopped() {
		logger.log(Level.FINEST, "firing coreStopped event");
		for (CoreListener coreListener : getListeners()) {
			coreListener.coreStopped();
		}
	}

	/**
	 * Notifies all listeners that a node was added to the core.
	 *
	 * @param node
	 *            The node that was added
	 */
	public void fireNodeAdded(Node node) {
		logger.log(Level.FINEST, "firing nodeAdded event with [node=" + node + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.nodeAdded(node);
		}
	}

	/**
	 * Notifies all listeners that a node was removed from the core.
	 *
	 * @param node
	 *            The node that was removed
	 */
	public void fireNodeRemoved(Node node) {
		logger.log(Level.FINEST, "firing nodeRemoved event with [node=" + node + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.nodeRemoved(node);
		}
	}

	/**
	 * Notifies all listeners that a connection to the given node is now being
	 * established.
	 *
	 * @param node
	 *            The node that is being connected to
	 */
	public void fireNodeConnecting(Node node) {
		logger.log(Level.FINEST, "firing nodeConnecting event with [node=" + node + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.nodeConnecting(node);
		}
	}

	/**
	 * Notifies all listeners that the given node is now connected.
	 *
	 * @param node
	 *            The node that is now connected
	 */
	public void fireNodeConnected(Node node) {
		logger.log(Level.FINEST, "firing nodeConnected event with [node=" + node + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.nodeConnected(node);
		}
	}

	/**
	 * Notifies all listeners that a connection to a node has failed.
	 *
	 * @param node
	 *            The node that could not be connected
	 * @param cause
	 *            The cause of the failure
	 */
	public void fireNodeConnectionFailed(Node node, Throwable cause) {
		logger.log(Level.FINEST, "firing nodeConnectionFailed event with [node=" + node + ",cause=" + cause + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.nodeConnectionFailed(node, cause);
		}
	}

	/**
	 * Notifies all listeners that the given node was disconnected.
	 *
	 * @param node
	 *            The node that is now disconnected
	 * @param throwable
	 *            The exception that caused the disconnect, or <code>null</code>
	 *            if there was no exception
	 */
	public void fireNodeDisconnected(Node node, Throwable throwable) {
		logger.log(Level.FINEST, "firing nodeDisconnected event with [node=" + node + ",throwable=" + throwable + "]");
		for (CoreListener coreListener : getListeners()) {
			coreListener.nodeDisconnected(node, throwable);
		}
	}

}
